<?php
/**
 * Plugin Name: Cart For WooCommerce Pro
 * Plugin URI: https://funnelkit.com/
 * Description: A WooCommerce Cart plugin from FunnelKit.
 * Version: 0.9.0
 * Author: FunnelKit
 */

namespace FKCart\Pro;
#[\AllowDynamicProperties]
class Plugin {
	private static $instance = null;

	private function __construct() {
		add_action( 'funnelkit_cart_loaded', [ $this, 'include_core' ], 15 );
	}

	public function include_core() {
		if ( ! class_exists( 'WFFN_Core' ) ) {
			/** If no FB lite plugin found */
			return;
		}

		include __DIR__ . '/include/upsells.php';
		include __DIR__ . '/include/rewards.php';

		Upsells::getInstance();
		Rewards::getInstance();
	}

	/**
	 * @return Plugin
	 */
	public static function getInstance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	public static function valid_l() {
		if ( ! method_exists( \WFFN_Core()->admin, 'get_license_config' ) ) {
			return true;
		}
		$License               = \WooFunnels_licenses::get_instance();
		$License->plugins_list = null;
		$License->get_plugins_list();
		$license_config = \WFFN_Core()->admin->get_license_config();
		$current        = new \DateTime( current_time( 'mysql', true ) );
		if ( ! isset( $license_config['gp'] ) ) {
			$license_config['gp'] = [ 7, 3 ];
		}
		if ( ! empty( $license_config['f']['ed'] ) ) {
			$expiry = new \DateTime( $license_config['f']['ed'] );

			$diff_in_days = $expiry->diff( $current )->format( "%a" );

			/**
			 * the expiry should always be less than on current utc
			 * and the difference should be less than days for grace period
			 * if both conditions hold true, then we need to stop returning order bumps
			 */
			if ( $expiry->getTimestamp() < $current->getTimestamp() && absint( $diff_in_days ) >= $license_config['gp'][0] ) {
				return false;
			}
		}

		/**
		 * if license not active, then we must check for activation date
		 */
		if ( true !== $license_config['f']['la'] && ! empty( $license_config['f']['ad'] ) ) {
			$activation_date = new \DateTime( $license_config['f']['ad'] );

			$diff_in_days = $activation_date->diff( $current )->format( "%a" );
			/**
			 * the activation should always be less than on current utc
			 * and the difference should be less than days for grace period
			 *  if both conditions hold true, then we need to stop returning order bumps
			 */
			if ( $activation_date->getTimestamp() < $current->getTimestamp() && absint( $diff_in_days ) >= $license_config['gp'][1] ) {
				return false;
			}
		}

		return true;
	}
}

Plugin::getInstance();
