<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/*
 * plugin Name: WooPayments by Automattic (v.7.8.1)
 *
 */

#[AllowDynamicProperties]
class WFACP_Compatibility_With_WooCommerce_Payments {
	public function __construct() {
		add_action( 'wfacp_internal_css', [ $this, 'enqueue_scripts' ] );
		add_action( 'wfacp_outside_header', [ $this, 'detect_woo_payment' ] );
		add_filter( 'wfacp_product_switcher_price_data', [ $this, 'wfacp_product_switcher_price_data' ], 10, 2 );
	}

	/*
	 * This code only work if woo_payment enabled from WooCommerce Settings
	 */
	public function detect_woo_payment() {
		$instance = WFACP_Common::remove_actions( 'woocommerce_checkout_billing', 'WC_Payments', 'woopay_fields_before_billing_details' );

		if ( $instance == 'WC_Payments' ) {
			add_action( 'wfacp_internal_css', [ $this, 'css' ] );
			add_filter( 'woocommerce_form_field_args', [ $this, 'add_aero_basic_classes' ], 10, 2 );
		}

	}

	public function css() {
		echo "<style>div#contact_details > h3{display:none}div#contact_details {clear: both;}</style>";
	}

	public function add_aero_basic_classes( $field, $key ) {
		if ( $key === 'billing_email' ) {
			$field['input_class'][] = 'wfacp-form-control';
			$tmp                    = [];
			if ( isset( $field['class'] ) && is_array( $field['class'] ) ) {
				$tmp = $field['class'];
			}
			$field['class']         = array_merge( [ 'woopay-billing-email' ], $tmp );
			$field['label_class'][] = 'wfacp-form-control-label';
		}

		return $field;
	}

	public function enqueue_scripts() {
		if ( is_null( WC()->cart ) || WC()->cart->needs_payment() ) {
			return;
		}
		$gateways = WC()->payment_gateways()->get_available_payment_gateways();

		if ( ! isset( $gateways['woocommerce_payments'] ) ) {
			return;
		}

		$gateway = $gateways['woocommerce_payments'];

		/**
		 * @var $gateway WC_Payment_Gateway_WCPay
		 */
		if ( method_exists( $gateway, 'get_payment_fields_js_config' ) ) {
			wp_localize_script( 'wcpay-checkout', 'wcpay_config', $gateway->get_payment_fields_js_config() );
			wp_enqueue_script( 'wcpay-checkout' );
			wp_enqueue_style( 'wcpay-checkout', plugins_url( 'dist/checkout.css', WCPAY_PLUGIN_FILE ), [], WC_Payments::get_file_version( 'dist/checkout.css' ) );
		}

	}

	/**
	 * @param $price_data
	 * @param $pro WC_Product;
	 *
	 * @return mixed
	 */
	public function wfacp_product_switcher_price_data( $price_data, $pro ) {

		$price_data['regular_org'] = $pro->get_regular_price();
		$price_data['price']       = $pro->get_price();

		return $price_data;
	}

}

WFACP_Plugin_Compatibilities::register( new WFACP_Compatibility_With_WooCommerce_Payments(), 'woocommerce_checkout' );

