<?php
defined( 'ABSPATH' ) || exit; //Exit if accessed directly

/**
 * This class will assign a course to optin prospects if the Learndash plugin is installed and free course is setup to assign
 * Class WFFN_Optin_Action_AffiliateWP_Lead
 */

include_once( 'integrations/class-wffn-optin-integration-affiliatewp.php' );

if ( ! class_exists( 'WFFN_Optin_Action_AffiliateWP_Lead' ) ) {
	#[AllowDynamicProperties]

class WFFN_Optin_Action_AffiliateWP_Lead extends WFFN_Optin_Action {

		private static $slug = 'affiliatewp_lead';
		private static $ins = null;
		public $priority = 30;

		/**
		 * WFFN_Optin_Action_AffiliateWP_Lead constructor.
		 */
		public function __construct() {
			parent::__construct();
			add_filter( 'wffn_custom_integration_field_merge', [ $this, 'afwp_merge_field' ], 10, 2 );
		}

		/**
		 * @return WFFN_Optin_Action_AffiliateWP_Lead|null
		 */
		public static function get_instance() {
			if ( null === self::$ins ) {
				self::$ins = new self;
			}

			return self::$ins;
		}

		/**
		 * @return bool
		 */
		public function should_register() {
			if ( class_exists( 'Affiliate_WP' ) ) {
				return true;
			}

			return false;
		}

		/**
		 * @return string
		 */
		public static function get_slug() {
			return self::$slug;
		}

		/**
		 * @param $posted_data
		 * @param $fields_settings
		 * @param $optin_action_settings
		 *
		 * @return array|bool|mixed
		 */
		public function handle_action( $posted_data, $fields_settings, $optin_action_settings ) {



			// Skip if AffiliateWP not active or Settings not enabled from backend.
			if ( ! ( $this->should_register() && ! empty( $optin_action_settings['affiliatewp_id'] ) && true === wffn_string_to_bool( $optin_action_settings['affiliatewp_id'] ) ) ) {

				return $posted_data;
			}

			$posted_data = parent::handle_action( $posted_data, $fields_settings, $optin_action_settings );

			$affiliate_id = affiliate_wp()->tracking->get_affiliate_id();

			// Skip if Affiliate ID does not exist.
			if ( empty( $affiliate_id ) ) {
				WFFN_Core()->logger->log( 'AffiliateID not captured!' );

				return $posted_data;
			}
			$afwp_integration = WFFN_Optin_Integration_AffiliateWP::get_instance();

			// Filter to change referral type ( 'opt-n', 'lead' ).
			$afwp_integration->referral_type = apply_filters( 'wffn_affiliatewp_integration_referral_type', 'opt-in' );

			$optin_page = get_the_title( $posted_data['optin_page_id'] );

			$description = $optin_page;

			$referral_total = 0;
			$afwp_integration->insert_pending_referral( $referral_total, $posted_data['optin_entry_id'], $description, [], [ 'affiliate_id' => $affiliate_id ] );

			return $posted_data;
		}

		public function afwp_merge_field( $fields, $optin_page_id ) {

			if ( ! empty( $fields ) && ! empty( $optin_page_id ) ) {
				$optin_actions_settings = WFOPP_Core()->optin_actions->get_optin_action_settings( $optin_page_id );
				if ( $this->should_register() && ! empty( $optin_actions_settings['affiliatewp_id'] ) && true === wffn_string_to_bool( $optin_actions_settings['affiliatewp_id'] )  ) {

					// GET Affiliate ID
					$affiliate_id = affiliate_wp()->tracking->get_affiliate_id();

					$affiliatewp_field_id = [
						'type'            => 'hidden',
						'label'           => 'Affiliate ID',
						'placeholder'     => null,
						'width'           => 'wffn-sm-100',
						'required'        => null,
						'InputName'       => 'Affiliate_ID',
						'default'         => $affiliate_id,
						'options'         => null,
						'radio_alignment' => 'horizontal',
					];

					array_push( $fields, $affiliatewp_field_id );
				} else {
					WFFN_Core()->logger->log( 'AffiliateWP Field rendering failed. Integration not set up correctly!' );
				}
			}

			return $fields;
		}

	}

	if ( class_exists( 'WFOPP_Core' ) ) {
		WFOPP_Core()->optin_actions->register( WFFN_Optin_Action_AffiliateWP_Lead::get_instance() );
	}

}
